@@ -1,64 +1,80 @@
-"""Basic data structures used to represent the Ledger entries.
-"""
-__copyright__ = "Copyright (C) 2013-2017  Martin Blais"
+"""Basic data structures used to represent the Ledger entries."""
+
+from __future__ import annotations
+
+__copyright__ = "Copyright (C) 2013-2022, 2024  Martin Blais"
 __license__ = "GNU GPLv2"
 
 import builtins
 import datetime
 import enum
 import sys
-
 from decimal import Decimal
-from typing import Any, Dict, FrozenSet, List, NamedTuple, Optional, Set, Union
+from typing import Any
+from typing import Iterator
+from typing import NamedTuple
+from typing import Protocol
+from typing import Union
+from typing import overload
 
+from beancount.core.account import has_component
 from beancount.core.amount import Amount
 from beancount.core.number import D
 from beancount.core.position import Cost
 from beancount.core.position import CostSpec
-from beancount.core.account import has_component
 from beancount.utils.bisect_key import bisect_left_with_key
 
-
 # Type declarations.
 Account = str
 Currency = str
 Flag = str
-Meta = Dict[str, Any]
+Meta = dict[str, Any]
+
+
+class BeancountError(Protocol):
+    """Beancount errors are objects with these attributes"""
+
+    @property
+    def source(self) -> Meta: ...
+    @property
+    def message(self) -> str: ...
+    @property
+    def entry(self) -> Directive | None: ...
 
 
 # An immutable constant for all empty sets. This is used to set links and tags
 # and ensure that they never has a None value. This makes some of the processing
 # code a bit simpler.
-EMPTY_SET = frozenset()
+EMPTY_SET: frozenset[str] = frozenset()
 
 
 # A set of valid booking method names for positions on accounts.
 # See http://furius.ca/beancount/doc/inventories for a full explanation.
 @enum.unique
 class Booking(enum.Enum):
-
     # Reject ambiguous matches with an error.
-    STRICT = 'STRICT'
+    STRICT = "STRICT"
 
     # Strict booking method, but disambiguate further with sizes. Reject
     # ambiguous matches with an error but if a lot matches the size exactly,
     # accept it the oldest.
-    STRICT_WITH_SIZE = 'STRICT_WITH_SIZE'
+    STRICT_WITH_SIZE = "STRICT_WITH_SIZE"
 
     # Disable matching and accept the creation of mixed inventories.
-    NONE = 'NONE'
+    NONE = "NONE"
 
     # Average cost booking: merge all matching lots before and after.
-    AVERAGE = 'AVERAGE'
+    AVERAGE = "AVERAGE"
 
     # First-in first-out in the case of ambiguity.
-    FIFO = 'FIFO'
+    FIFO = "FIFO"
 
     # Last-in first-out in the case of ambiguity.
-    LIFO = 'LIFO'
+    LIFO = "LIFO"
 
     # Highest-in first-out in the case of ambiguity.
-    HIFO = 'HIFO'
+    HIFO = "HIFO"
+
 
 # All possible types of entries. These are the main data structures in use
 # within the program. They are all treated as immutable.
@@ -90,11 +106,12 @@ class Open(NamedTuple):
         be should be left unspecified (None) in the vast majority of cases. See
         Booking below for a selection of valid methods.
     """
+
     meta: Meta
     date: datetime.date
     account: Account
-    currencies: List[Currency]
-    booking: Optional[Booking]
+    currencies: list[Currency]
+    booking: Booking | None
 
 
 class Close(NamedTuple):
@@ -106,6 +123,7 @@ class Close(NamedTuple):
       date: See above.
       account: A string, the name of the account that is being closed.
     """
+
     meta: Meta
     date: datetime.date
     account: Account
@@ -127,6 +145,7 @@ class Commodity(NamedTuple):
       date: See above.
       currency: A string, the commodity under consideration.
     """
+
     meta: Meta
     date: datetime.date
     currency: Currency
@@ -147,6 +166,7 @@ class Pad(NamedTuple):
       source_account: A string, the name of the account which is used to debit from
         in order to fill 'account'.
     """
+
     meta: Meta
     date: datetime.date
     account: Account
@@ -173,12 +193,13 @@ class Balance(NamedTuple):
       tolerance: A Decimal object, the amount of tolerance to use in the
         verification.
     """
+
     meta: Meta
     date: datetime.date
     account: Account
     amount: Amount
-    tolerance: Optional[Decimal]
-    diff_amount: Optional[Amount]
+    tolerance: Decimal | None
+    diff_amount: Amount | None
 
 
 class Posting(NamedTuple):
@@ -205,12 +226,13 @@ class Posting(NamedTuple):
         specifically to that posting, or None, if not provided. In practice, most
         of the instances will be unlikely to have metadata.
     """
+
     account: Account
-    units: Optional[Amount]
-    cost: Optional[Union[Cost, CostSpec]]
-    price: Optional[Amount]
-    flag: Optional[Flag]
-    meta: Optional[Meta]
+    units: Amount | None
+    cost: Cost | CostSpec | None
+    price: Amount | None
+    flag: Flag | None
+    meta: Meta | None
 
 
 class Transaction(NamedTuple):
@@ -235,14 +257,15 @@ class Transaction(NamedTuple):
       postings: A list of Posting instances, the legs of this transaction. See the
         doc under Posting above.
     """
+
     meta: Meta
     date: datetime.date
     flag: Flag
-    payee: Optional[str]
-    narration: str
-    tags: FrozenSet
-    links: FrozenSet
-    postings: List[Posting]
+    payee: str | None
+    narration: str | None
+    tags: frozenset[str]
+    links: frozenset[str]
+    postings: list[Posting]
 
 
 class TxnPosting(NamedTuple):
@@ -255,6 +278,7 @@ class TxnPosting(NamedTuple):
       txn: The parent Transaction instance.
       posting: The Posting instance.
     """
+
     txn: Transaction
     posting: Posting
 
@@ -276,12 +300,13 @@ class Note(NamedTuple):
       comment: A free-form string, the text of the note. This can be long if you
         want it to.
     """
+
     meta: Meta
     date: datetime.date
     account: Account
     comment: str
-    tags: Optional[Set]
-    links: Optional[Set]
+    tags: frozenset[str] | None
+    links: frozenset[str] | None
 
 
 class Event(NamedTuple):
@@ -314,6 +339,7 @@ class Event(NamedTuple):
       description: A free-form string, the value of the variable as of the date
         of the transaction.
     """
+
     meta: Meta
     date: datetime.date
     type: str
@@ -335,6 +361,7 @@ class Query(NamedTuple):
       name: A string, the unique identifier for the query.
       query_string: The SQL query string to be run or made available.
     """
+
     meta: Meta
     date: datetime.date
     name: str
@@ -359,6 +386,7 @@ class Price(NamedTuple):
      amount: An instance of Amount, the number of units and currency that
        'currency' is worth, for instance 1200.12 USD.
     """
+
     meta: Meta
     date: datetime.date
     currency: Currency
@@ -386,12 +414,13 @@ class Document(NamedTuple):
       tags: A set of tag strings (without the '#'), or None, if an empty set.
       links: A set of link strings (without the '^'), or None, if an empty set.
     """
+
     meta: Meta
     date: datetime.date
     account: Account
     filename: str
-    tags: Optional[Set]
-    links: Optional[Set]
+    tags: frozenset[str] | None
+    links: frozenset[str] | None
 
 
 class Custom(NamedTuple):
@@ -411,10 +440,11 @@ class Custom(NamedTuple):
         (Note that this list is not enforced to be consistent for all directives
         of the same type by the parser.)
     """
+
     meta: Meta
     date: datetime.date
     type: str
-    values: List
+    values: list[Any]
 
 
 # A list of all the valid directive types.
@@ -430,7 +460,7 @@ ALL_DIRECTIVES = (
     Query,
     Price,
     Document,
-    Custom
+    Custom,
 )
 
 # Type for any of the directives.
@@ -446,12 +476,13 @@ Directive = Union[
     Query,
     Price,
     Document,
-    Custom
+    Custom,
 ]
 
 
-class dtypes:  # pylint: disable=invalid-name
+class dtypes:
     "Types of directives."
+
     Open = Open
     Close = Close
     Commodity = Commodity
@@ -468,12 +499,12 @@ class dtypes:  # pylint: disable=invalid-name
 
 # Type for the list of entries and options map.
 # Note: In v3, make the terminology 'Entries' obsolete.
-Entries = List[Directive]
-Directives = List[Directive]
-Options = Dict[str, Any]
+Entries = list[Directive]
+Directives = list[Directive]
+Options = dict[str, Any]
 
 
-def new_metadata(filename, lineno, kvlist=None):
+def new_metadata(filename: str, lineno: int, kvlist: Meta | None = None) -> Meta:
     """Create a new metadata container from the filename and line number.
 
     Args:
@@ -483,14 +514,28 @@ def new_metadata(filename, lineno, kvlist=None):
     Returns:
       A metadata dict.
     """
-    meta = {'filename': filename,
-            'lineno': lineno}
+    meta = {"filename": filename, "lineno": lineno}
     if kvlist:
         meta.update(kvlist)
     return meta
 
 
-def create_simple_posting(entry, account, number, currency):
+# Either both number and currency should be provided or both should be None
+@overload
+def create_simple_posting(
+    entry: Transaction, account: Account, number: Decimal | str, currency: str
+) -> Posting: ...
+
+
+@overload
+def create_simple_posting(
+    entry: Transaction, account: Account, number: None, currency: None
+) -> Posting: ...
+
+
+def create_simple_posting(
+    entry: Transaction, account: Account, number: Decimal | str | None, currency: str | None
+) -> Posting:
     """Create a simple posting on the entry, with just a number and currency (no cost).
 
     Args:
@@ -504,7 +549,7 @@ def create_simple_posting(entry, account, number, currency):
     """
     if isinstance(account, str):
         pass
-    if number is None:
+    if number is None or currency is None:
         units = None
     else:
         if not isinstance(number, Decimal):
@@ -516,10 +561,15 @@ def create_simple_posting(entry, account, number, currency):
     return posting
 
 
-def create_simple_posting_with_cost(entry, account,
-                                    number, currency,
-                                    cost_number, cost_currency):
-    """Create a simple posting on the entry, with just a number and currency (no cost).
+def create_simple_posting_with_cost(
+    entry: Transaction,
+    account: Account,
+    number: Decimal | str,
+    currency: str,
+    cost_number: Decimal | str,
+    cost_currency: str,
+) -> Posting:
+    """Create a simple posting on the entry, with a number and currency and cost.
 
     Args:
       entry: The entry instance to add the posting to.
@@ -536,19 +586,22 @@ def create_simple_posting_with_cost(entry, account,
         pass
     if not isinstance(number, Decimal):
         number = D(number)
-    if cost_number and not isinstance(cost_number, Decimal):
+    if cost_number is not None and not isinstance(cost_number, Decimal):
         cost_number = D(cost_number)
     units = Amount(number, currency)
-    cost = Cost(cost_number, cost_currency, None, None)
+    cost = Cost(cost_number, cost_currency, datetime.date(1, 1, 1), None)
     posting = Posting(account, units, cost, None, None, None)
     if entry is not None:
         entry.postings.append(posting)
     return posting
 
 
-NoneType = type(None)
+NoneType: type = type(None)
 
-def sanity_check_types(entry, allow_none_for_tags_and_links=False):
+
+def sanity_check_types(
+    entry: Directive, allow_none_for_tags_and_links: bool = False
+) -> None:
     """Check that the entry and its postings has all correct data types.
 
     Args:
@@ -561,20 +614,24 @@ def sanity_check_types(entry, allow_none_for_tags_and_links=False):
     """
     assert isinstance(entry, ALL_DIRECTIVES), "Invalid directive type"
     assert isinstance(entry.meta, dict), "Invalid type for meta"
-    assert 'filename' in entry.meta, "Missing filename in metadata"
-    assert 'lineno' in entry.meta, "Missing line number in metadata"
+    assert "filename" in entry.meta, "Missing filename in metadata"
+    assert "lineno" in entry.meta, "Missing line number in metadata"
     assert isinstance(entry.date, datetime.date), "Invalid date type"
     if isinstance(entry, Transaction):
-        assert isinstance(entry.flag, (NoneType, str)), "Invalid flag type"
+        assert isinstance(entry.flag, str), "Invalid flag type"
         assert isinstance(entry.payee, (NoneType, str)), "Invalid payee type"
         assert isinstance(entry.narration, (NoneType, str)), "Invalid narration type"
-        set_types = ((NoneType, set, frozenset)
-                     if allow_none_for_tags_and_links
-                     else (set, frozenset))
-        assert isinstance(entry.tags, set_types), (
-            "Invalid tags type: {}".format(type(entry.tags)))
-        assert isinstance(entry.links, set_types), (
-            "Invalid links type: {}".format(type(entry.links)))
+        set_types = (
+            (NoneType, set, frozenset)
+            if allow_none_for_tags_and_links
+            else (set, frozenset)
+        )
+        assert isinstance(entry.tags, set_types), "Invalid tags type: {}".format(
+            type(entry.tags)
+        )
+        assert isinstance(entry.links, set_types), "Invalid links type: {}".format(
+            type(entry.links)
+        )
         assert isinstance(entry.postings, list), "Invalid postings list type"
         for posting in entry.postings:
             assert isinstance(posting, Posting), "Invalid posting type"
@@ -585,7 +642,7 @@ def sanity_check_types(entry, allow_none_for_tags_and_links=False):
             assert isinstance(posting.flag, (str, NoneType)), "Invalid flag type"
 
 
-def posting_has_conversion(posting):
+def posting_has_conversion(posting: Posting) -> bool:
     """Return true if this position involves a conversion.
 
     A conversion is when there is a price attached to the amount but no cost.
@@ -596,11 +653,10 @@ def posting_has_conversion(posting):
     Return:
       A boolean, true if this posting has a price conversion.
     """
-    return (posting.cost is None and
-            posting.price is not None)
+    return posting.cost is None and posting.price is not None
 
 
-def transaction_has_conversion(transaction):
+def transaction_has_conversion(transaction: Transaction) -> bool:
     """Given a Transaction entry, return true if at least one of
     the postings has a price conversion (without an associated
     cost). These are the source of non-zero conversion balances.
@@ -611,15 +667,16 @@ def transaction_has_conversion(transaction):
       A boolean, true if this transaction contains at least one posting with a
       price conversion.
     """
-    assert isinstance(transaction, Transaction), (
-        "Invalid type of entry for transaction: {}".format(transaction))
+    assert isinstance(
+        transaction, Transaction
+    ), "Invalid type of entry for transaction: {}".format(transaction)
     for posting in transaction.postings:
         if posting_has_conversion(posting):
             return True
     return False
 
 
-def get_entry(posting_or_entry):
+def get_entry(posting_or_entry: Directive | TxnPosting) -> Directive:
     """Return the entry associated with the posting or entry.
 
     Args:
@@ -627,9 +684,11 @@ def get_entry(posting_or_entry):
     Returns:
       A datetime instance.
     """
-    return (posting_or_entry.txn
-            if isinstance(posting_or_entry, TxnPosting)
-            else posting_or_entry)
+    return (
+        posting_or_entry.txn
+        if isinstance(posting_or_entry, TxnPosting)
+        else posting_or_entry
+    )
 
 
 # Sorting order of directives on the same day, by type:
@@ -645,7 +704,7 @@ def get_entry(posting_or_entry):
 SORT_ORDER = {Open: -2, Balance: -1, Document: 1, Close: 2}
 
 
-def entry_sortkey(entry):
+def entry_sortkey(entry: Directive) -> tuple[datetime.date, int, int]:
     """Sort-key for entries. We sort by date, except that checks
     should be placed in front of every list of entries of that same day,
     in order to balance linearly.
@@ -659,7 +718,7 @@ def entry_sortkey(entry):
     return (entry.date, SORT_ORDER.get(type(entry), 0), entry.meta["lineno"])
 
 
-def sorted(entries):
+def sorted(entries: Directives) -> Directives:
     """A convenience to sort a list of entries, using entry_sortkey().
 
     Args:
@@ -670,24 +729,25 @@ def sorted(entries):
     return builtins.sorted(entries, key=entry_sortkey)
 
 
-def posting_sortkey(entry):
-    """Sort-key for entries or postings. We sort by date, except that checks
-    should be placed in front of every list of entries of that same day,
-    in order to balance linearly.
+def posting_sortkey(entry: Directive | TxnPosting) -> tuple[datetime.date, int, int]:
+    """Sort-key for entries or TxnPosting instances. We sort by date, except
+    that checks should be placed in front of every list of entries of that same
+    day, in order to balance linearly.
 
     Args:
-      entry: A Posting or entry instance
+      entry: A TxnPosting or entry instance.
     Returns:
       A tuple of (date, integer, integer), that forms the sort key for the
-      posting or entry.
+      TxnPosting or entry.
     """
+    assert isinstance(entry, (TxnPosting,) + ALL_DIRECTIVES)
     if isinstance(entry, TxnPosting):
         entry = entry.txn
     return (entry.date, SORT_ORDER.get(type(entry), 0), entry.meta["lineno"])
 
 
 # TODO(blais): Rename 'txns' to 'transactions' for clarity.
-def filter_txns(entries):
+def filter_txns(entries: Directives) -> Iterator[Transaction]:
     """A generator that yields only the Transaction instances.
 
     This is such an incredibly common operation that it deserves a terse
@@ -703,7 +763,7 @@ def filter_txns(entries):
             yield entry
 
 
-def has_entry_account_component(entry, component):
+def has_entry_account_component(entry: Directive, component: str) -> bool:
     """Return true if one of the entry's postings has an account component.
 
     Args:
@@ -714,12 +774,12 @@ def has_entry_account_component(entry, component):
       Boolean: true if the component is in the account. Note that a component
       name must be whole, that is ``NY`` is not in ``Expenses:Taxes:StateNY``.
     """
-    return (isinstance(entry, Transaction) and
-            any(has_component(posting.account, component)
-                for posting in entry.postings))
+    return isinstance(entry, Transaction) and any(
+        has_component(posting.account, component) for posting in entry.postings
+    )
 
 
-def find_closest(entries, filename, lineno):
+def find_closest(entries: Directives, filename: str, lineno: int) -> Directive | None:
     """Find the closest entry from entries to (filename, lineno).
 
     Args:
@@ -745,7 +805,7 @@ def find_closest(entries, filename, lineno):
     return closest_entry
 
 
-def remove_account_postings(account, entries):
+def remove_account_postings(account: Account, entries: Directives) -> Directives:
     """Remove all postings with the given account.
 
     Args:
@@ -756,15 +816,20 @@ def remove_account_postings(account, entries):
     new_entries = []
     for entry in entries:
         if isinstance(entry, Transaction) and (
-                any(posting.account == account for posting in entry.postings)):
-            entry = entry._replace(postings=[posting
-                                             for posting in entry.postings
-                                             if posting.account != account])
+            any(posting.account == account for posting in entry.postings)
+        ):
+            entry = entry._replace(
+                postings=[
+                    posting for posting in entry.postings if posting.account != account
+                ]
+            )
         new_entries.append(entry)
     return new_entries
 
 
-def iter_entry_dates(entries, date_begin, date_end):
+def iter_entry_dates(
+    entries: Directives, date_begin: datetime.date, date_end: datetime.date
+) -> Iterator[Directive]:
     """Iterate over the entries in a date window.
 
     Args:
