@@ -15,7 +15,8 @@ final balance of that account, resulting from its list of postings.
 You should not build RealAccount trees yourself; instead, you should filter the
 list of desired directives to display and call the realize() function with them.
 """
-__copyright__ = "Copyright (C) 2013-2016  Martin Blais"
+
+__copyright__ = "Copyright (C) 2013-2021, 2024  Martin Blais"
 __license__ = "GNU GPLv2"
 
 import collections
@@ -24,21 +25,21 @@ import functools
 import io
 import operator
 
-from beancount.core.data import Transaction
-from beancount.core.data import Posting
-from beancount.core.data import TxnPosting
-from beancount.core.data import Balance
-from beancount.core.data import Open
-from beancount.core.data import Close
-from beancount.core.data import Pad
-from beancount.core.data import Note
-from beancount.core.data import Document
-from beancount.core.data import Custom
-from beancount.core import inventory
-from beancount.core import amount
-from beancount.core import data
 from beancount.core import account
+from beancount.core import amount
 from beancount.core import convert
+from beancount.core import data
+from beancount.core import inventory
+from beancount.core.data import Balance
+from beancount.core.data import Close
+from beancount.core.data import Custom
+from beancount.core.data import Document
+from beancount.core.data import Note
+from beancount.core.data import Open
+from beancount.core.data import Pad
+from beancount.core.data import Posting
+from beancount.core.data import Transaction
+from beancount.core.data import TxnPosting
 
 
 class RealAccount(dict):
@@ -50,7 +51,8 @@ class RealAccount(dict):
         include the postings of children accounts).
       balance: The final balance of the list of postings associated with this account.
     """
-    __slots__ = ('account', 'txn_postings', 'balance')
+
+    __slots__ = ("account", "txn_postings", "balance")
 
     def __init__(self, account_name, *args, **kwargs):
         """Create a RealAccount instance.
@@ -79,8 +81,11 @@ class RealAccount(dict):
         if not isinstance(value, RealAccount):
             raise ValueError("Invalid RealAccount value: '{}'".format(value))
         if not value.account.endswith(key):
-            raise ValueError("RealAccount name '{}' inconsistent with key: '{}'".format(
-                value.account, key))
+            raise ValueError(
+                "RealAccount name '{}' inconsistent with key: '{}'".format(
+                    value.account, key
+                )
+            )
         return super().__setitem__(key, value)
 
     def copy(self):
@@ -103,10 +108,12 @@ class RealAccount(dict):
         Returns:
           A boolean, True if the two real accounts are equal.
         """
-        return (dict.__eq__(self, other) and
-                self.account == other.account and
-                self.balance == other.balance and
-                self.txn_postings == other.txn_postings)
+        return (
+            dict.__eq__(self, other)
+            and self.account == other.account
+            and self.balance == other.balance
+            and self.txn_postings == other.txn_postings
+        )
 
     def __ne__(self, other):
         """Not-equality predicate. See __eq__.
@@ -258,7 +265,7 @@ def realize(entries, min_accounts=None, compute_balance=True):
     txn_postings_map = postings_by_account(entries)
 
     # Create a RealAccount tree and compute the balance for each.
-    real_root = RealAccount('')
+    real_root = RealAccount("")
     for account_name, txn_postings in txn_postings_map.items():
         real_account = get_or_create(real_root, account_name)
         real_account.txn_postings = txn_postings
@@ -291,12 +298,10 @@ def postings_by_account(entries):
     """
     txn_postings_map = collections.defaultdict(list)
     for entry in entries:
-
         if isinstance(entry, Transaction):
             # Insert an entry for each of the postings.
             for posting in entry.postings:
-                txn_postings_map[posting.account].append(
-                    TxnPosting(entry, posting))
+                txn_postings_map[posting.account].append(TxnPosting(entry, posting))
 
         elif isinstance(entry, (Open, Close, Balance, Note, Document)):
             # Append some other entries in the realized list.
@@ -417,7 +422,6 @@ def iterate_with_balance(txn_postings):
 
     first = lambda pair: pair[0]
     for txn_posting in txn_postings:
-
         # Get the posting if we are dealing with one.
         assert not isinstance(txn_posting, Posting)
         if isinstance(txn_posting, TxnPosting):
@@ -428,8 +432,9 @@ def iterate_with_balance(txn_postings):
             entry = txn_posting
 
         if entry.date != prev_date:
-            assert prev_date is None or entry.date > prev_date, (
-                "Invalid date order for postings: {} > {}".format(prev_date, entry.date))
+            assert (
+                prev_date is None or entry.date > prev_date
+            ), "Invalid date order for postings: {} > {}".format(prev_date, entry.date)
             prev_date = entry.date
 
             # Flush the dated entries.
@@ -480,8 +485,9 @@ def compute_balance(real_account, leaf_only=False):
     Returns:
       An Inventory.
     """
-    return functools.reduce(operator.add, [
-        ra.balance for ra in iter_children(real_account, leaf_only)])
+    return functools.reduce(
+        operator.add, [ra.balance for ra in iter_children(real_account, leaf_only)]
+    )
 
 
 def find_last_active_posting(txn_postings):
@@ -547,13 +553,13 @@ def dump(root_account):
     # Start with the root node. We push the constant prefix before this node,
     # the account name, and the RealAccount instance. We will maintain a stack
     # of children nodes to render.
-    stack = [('', root_account.account, root_account, True)]
+    stack = [("", root_account.account, root_account, True)]
     while stack:
         prefix, name, real_account, is_last = stack.pop(-1)
 
         if real_account is root_account:
             # For the root node, we don't want to render any prefix.
-            first = cont = ''
+            first = cont = ""
         else:
             # Compute the string that precedes the name directly and the one below
             # that for the continuation lines.
@@ -580,9 +586,7 @@ def dump(root_account):
 
         # Add a line for this account.
         if not (real_account is root_account and not name):
-            lines.append((first + name,
-                          cont + cont_name,
-                          real_account))
+            lines.append((first + name, cont + cont_name, real_account))
 
         # Push the children onto the stack, being careful with ordering and
         # marking the last node as such.
@@ -600,17 +604,17 @@ def dump(root_account):
     # Compute the maximum width of the lines and convert all of them to the same
     # maximal width. This makes it easy on the client.
     max_width = max(len(first_line) for first_line, _, __ in lines)
-    line_format = '{{:{width}}}'.format(width=max_width)
-    return [(line_format.format(first_line),
-             line_format.format(cont_line),
-             real_node)
-            for (first_line, cont_line, real_node) in lines]
+    line_format = "{{:{width}}}".format(width=max_width)
+    return [
+        (line_format.format(first_line), line_format.format(cont_line), real_node)
+        for (first_line, cont_line, real_node) in lines
+    ]
 
 
-PREFIX_CHILD_1 = '|-- '
-PREFIX_CHILD_C = '|   '
-PREFIX_LEAF_1 = '`-- '
-PREFIX_LEAF_C = '    '
+PREFIX_CHILD_1 = "|-- "
+PREFIX_CHILD_C = "|   "
+PREFIX_LEAF_1 = "`-- "
+PREFIX_LEAF_C = "    "
 
 
 def dump_balances(real_root, dformat, at_cost=False, fullnames=False, file=None):
@@ -629,11 +633,13 @@ def dump_balances(real_root, dformat, at_cost=False, fullnames=False, file=None)
     """
     if fullnames:
         # Compute the maximum account name length;
-        maxlen = max(len(real_child.account)
-                     for real_child in iter_children(real_root, leaf_only=True))
-        line_format = '{{:{width}}} {{}}\n'.format(width=maxlen)
+        maxlen = max(
+            len(real_child.account)
+            for real_child in iter_children(real_root, leaf_only=True)
+        )
+        line_format = "{{:{width}}} {{}}\n".format(width=maxlen)
     else:
-        line_format = '{}       {}\n'
+        line_format = "{}       {}\n"
 
     output = file or io.StringIO()
     for first_line, cont_line, real_account in dump(real_root):
@@ -643,10 +649,12 @@ def dump_balances(real_root, dformat, at_cost=False, fullnames=False, file=None)
             else:
                 rinv = real_account.balance.reduce(convert.get_units)
             amounts = [position.units for position in rinv.get_positions()]
-            positions = [amount_.to_string(dformat)
-                         for amount_ in sorted(amounts, key=amount.sortkey)]
+            positions = [
+                amount_.to_string(dformat)
+                for amount_ in sorted(amounts, key=amount.sortkey)
+            ]
         else:
-            positions = ['']
+            positions = [""]
 
         if fullnames:
             for position in positions:
