@@ -1,78 +1,109 @@
-"""Builder for Beancount grammar.
-"""
-__copyright__ = "Copyright (C) 2015-2016  Martin Blais"
+"""Builder for Beancount grammar."""
+
+from __future__ import annotations
+
+__copyright__ = "Copyright (C) 2013-2021, 2023-2024  Martin Blais"
 __license__ = "GNU GPLv2"
 
 import collections
 import copy
 import traceback
-from os import path
 from datetime import date
 from decimal import Decimal
+from os import path
+from typing import Any
+from typing import NamedTuple
 
 import regex
 
-from beancount.core.number import ZERO
-from beancount.core.number import MISSING
-from beancount.core.amount import Amount
-from beancount.core import display_context
-from beancount.core.position import CostSpec
-from beancount.core.data import Transaction
-from beancount.core.data import Balance
-from beancount.core.data import Open
-from beancount.core.data import Close
-from beancount.core.data import Commodity
-from beancount.core.data import Pad
-from beancount.core.data import Event
-from beancount.core.data import Query
-from beancount.core.data import Price
-from beancount.core.data import Note
-from beancount.core.data import Document
-from beancount.core.data import Custom
-from beancount.core.data import new_metadata
-from beancount.core.data import Posting
-from beancount.core.data import Booking
-from beancount.core.data import EMPTY_SET
-
-from beancount.parser import lexer
-from beancount.parser import options
 from beancount.core import account
 from beancount.core import data
+from beancount.core import display_context
+from beancount.core.amount import Amount
+from beancount.core.data import EMPTY_SET
+from beancount.core.data import Balance
+from beancount.core.data import Booking
+from beancount.core.data import Close
+from beancount.core.data import Commodity
+from beancount.core.data import Custom
+from beancount.core.data import Document
+from beancount.core.data import Event
+from beancount.core.data import Meta
+from beancount.core.data import Note
+from beancount.core.data import Open
+from beancount.core.data import Pad
+from beancount.core.data import Posting
+from beancount.core.data import Price
+from beancount.core.data import Query
+from beancount.core.data import Transaction
+from beancount.core.data import new_metadata
+from beancount.core.number import MISSING
+from beancount.core.number import ZERO
+from beancount.core.position import CostSpec
+from beancount.parser import lexer
+from beancount.parser import options
 
 
-ParserError = collections.namedtuple('ParserError', 'source message entry')
-ParserSyntaxError = collections.namedtuple('ParserSyntaxError', 'source message entry')
-DeprecatedError = collections.namedtuple('DeprecatedError', 'source message entry')
+class ParserError(NamedTuple):
+    source: Meta
+    message: str
+    entry: None = None
 
 
+class ParserSyntaxError(NamedTuple):
+    source: Meta
+    message: str
+    entry: None = None
 
-# Key-value pairs. This is used to hold meta-data attachments temporarily.
-#
-# Attributes:
-#  key: A string, the name of the key.
-#  value: Any object.
-KeyValue = collections.namedtuple('KeyValue', 'key value')
 
-# Value-type pairs. This is used to represent custom values where the concrete
-# datatypes aren't matching those which are found in the parser.
-#
-# Attributes:
-#  value: Any object.
-#  dtype: The datatype of the object.
-ValueType = collections.namedtuple('ValueType', 'value dtype')
+class DeprecatedError(NamedTuple):
+    source: Meta
+    message: str
+    entry: None = None
 
-# Convenience holding class for amounts with per-share and total value.
-#
-# Attributes:
-#   number_per: A Decimal instance, the cost/price per unit.
-#   number_total: A Decimal instance, the total cost/price.
-#   currency: A string, the commodity of the amount.
-CompoundAmount = collections.namedtuple('CompoundAmount',
-                                        'number_per number_total currency')
+
+class KeyValue(NamedTuple):
+    """Key-value pairs. This is used to hold meta-data attachments temporarily.
+
+    Attributes:
+      key: A string, the name of the key.
+      value: Any object.
+    """
+
+    key: str
+    value: Any
+
+
+class ValueType(NamedTuple):
+    """
+    Value-type pairs. This is used to represent custom values where the concrete
+    datatypes aren't matching those which are found in the parser.
+
+    Attributes:
+        value: Any object.
+        dtype: The datatype of the object.
+    """
+
+    value: Any
+    dtype: type
+
+
+class CompoundAmount(NamedTuple):
+    """Convenience holding class for amounts with per-share and total value.
+
+    Attributes:
+      number_per: A Decimal instance, the cost/price per unit.
+      number_total: A Decimal instance, the total cost/price.
+      currency: A string, the commodity of the amount.
+    """
+
+    number_per: Decimal
+    number_total: Decimal
+    currency: str
 
 
 # A unique token used to indicate a merge of the lots of an inventory.
-MERGE_COST = '***'
+MERGE_COST = "***"
 
 
 def valid_account_regexp(options):
@@ -83,36 +114,38 @@ def valid_account_regexp(options):
     Returns:
       A string, a regular expression that will match all account names.
     """
-    names = map(options.__getitem__, ('name_assets',
-                                      'name_liabilities',
-                                      'name_equity',
-                                      'name_income',
-                                      'name_expenses'))
+    names = map(
+        options.__getitem__,
+        ("name_assets", "name_liabilities", "name_equity", "name_income", "name_expenses"),
+    )
 
     # Replace the first term of the account regular expression with the specific
     # names allowed under the options configuration. This code is kept in sync
     # with {5672c7270e1e}.
-    return regex.compile("(?:{})(?:{}{})+".format('|'.join(names),
-                                                  account.sep,
-                                                  account.ACC_COMP_NAME_RE))
+    return regex.compile(
+        "(?:{})(?:{}{})+".format("|".join(names), account.sep, account.ACC_COMP_NAME_RE)
+    )
 
 
-# A temporary data structure used during parsing to hold and accumulate the
-# fields being parsed on a transaction line. Because we want to be able to parse
-# these in arbitrary order, we have to accumulate the fields and then unpack
-# them intelligently in the transaction callback.
-#
-# Attributes:
-#  tags: a set object  of the tags to be applied to this transaction.
-#  links: a set of link strings to be applied to this transaction.
-TagsLinks = collections.namedtuple('TagsLinks', 'tags links')
+class TagsLinks(NamedTuple):
+    """A temporary data structure used during parsing to hold and accumulate the
+    fields being parsed on a transaction line. Because we want to be able to parse
+    these in arbitrary order, we have to accumulate the fields and then unpack
+    them intelligently in the transaction callback.
+
+    Attributes:
+      tags: A set object  of the tags to be applied to this transaction.
+      links: A set of link strings to be applied to this transaction.
+    """
+
+    tags: set[str]
+    links: set[str]
 
 
 class Builder(lexer.LexBuilder):
     """A builder used by the lexer and grammar parser as callbacks to create
     the data objects corresponding to rules parsed from the input file."""
 
-    # pylint: disable=too-many-instance-attributes
     def __init__(self):
         lexer.LexBuilder.__init__(self)
 
@@ -155,21 +188,24 @@ class Builder(lexer.LexBuilder):
         """
         # If the user left some tags unbalanced, issue an error.
         for tag in self.tags:
-            meta = new_metadata(self.options['filename'], 0)
-            self.errors.append(
-                ParserError(meta, "Unbalanced pushed tag: '{}'".format(tag), None))
+            meta = new_metadata(self.options["filename"], 0)
+            self.errors.append(ParserError(meta, "Unbalanced pushed tag: '{}'".format(tag)))
 
         # If the user left some metadata unpopped, issue an error.
         for key, value_list in self.meta.items():
-            meta = new_metadata(self.options['filename'], 0)
+            meta = new_metadata(self.options["filename"], 0)
             self.errors.append(
-                ParserError(meta, (
-                    "Unbalanced metadata key '{}'; leftover metadata '{}'").format(
-                        key, ', '.join(value_list)), None))
+                ParserError(
+                    meta,
+                    ("Unbalanced metadata key '{}'; leftover metadata '{}'").format(
+                        key, ", ".join(value_list)
+                    ),
+                )
+            )
 
         # Weave the commas option in the DisplayContext itself, so it propagates
         # everywhere it is used automatically.
-        self.dcontext.set_commas(self.options['render_commas'])
+        self.dcontext.set_commas(self.options["render_commas"])
 
         return (self.get_entries(), self.errors, self.get_options())
 
@@ -188,13 +224,13 @@ class Builder(lexer.LexBuilder):
           A dict of option names to options.
         """
         # Build and store the inferred DisplayContext instance.
-        self.options['dcontext'] = self.dcontext
+        self.options["dcontext"] = self.dcontext
 
         return self.options
 
     def get_long_string_maxlines(self):
         """See base class."""
-        return self.options['long_string_maxlines']
+        return self.options["long_string_maxlines"]
 
     def store_result(self, filename, lineno, entries):
         """Start rule stores the final result here.
@@ -205,10 +241,11 @@ class Builder(lexer.LexBuilder):
         if entries:
             self.entries = entries
         # Also record the name of the processed file.
-        self.options['filename'] = filename
+        self.options["filename"] = filename
 
-    def build_grammar_error(self, filename, lineno, exc_value,
-                            exc_type=None, exc_traceback=None):
+    def build_grammar_error(
+        self, filename, lineno, exc_value, exc_type=None, exc_traceback=None
+    ):
         """Build a grammar error and appends it to the list of pending errors.
 
         Args:
@@ -223,12 +260,11 @@ class Builder(lexer.LexBuilder):
             strings = traceback.format_exception_only(exc_type, exc_value)
             tblist = traceback.extract_tb(exc_traceback)
             filename, lineno, _, __ = tblist[0]
-            message = '{} ({}:{})'.format(strings[0], filename, lineno)
+            message = "{} ({}:{})".format(strings[0], filename, lineno)
         else:
             message = str(exc_value)
         meta = new_metadata(filename, lineno)
-        self.errors.append(
-            ParserSyntaxError(meta, message, None))
+        self.errors.append(ParserSyntaxError(meta, message))
 
     def account(self, filename, lineno, account):
         """Check account name validity.
@@ -241,7 +277,8 @@ class Builder(lexer.LexBuilder):
         if not self.account_regexp.match(account):
             meta = new_metadata(filename, lineno)
             self.errors.append(
-                ParserError(meta, "Invalid account name: {}".format(account), None))
+                ParserError(meta, "Invalid account name: {}".format(account))
+            )
         # Intern account names. This should reduces memory usage a
         # fair bit because these strings are repeated liberally.
         return self.accounts.setdefault(account, account)
@@ -253,11 +290,10 @@ class Builder(lexer.LexBuilder):
           filename: The current filename
           lineno: The current line number
         """
-        if self.options['allow_pipe_separator']:
+        if self.options["allow_pipe_separator"]:
             return
         meta = new_metadata(filename, lineno)
-        self.errors.append(
-            ParserSyntaxError(meta, "Pipe symbol is deprecated.", None))
+        self.errors.append(ParserSyntaxError(meta, "Pipe symbol is deprecated."))
 
     def pushtag(self, filename, lineno, tag):
         """Push a tag on the current set of tags.
@@ -280,7 +316,8 @@ class Builder(lexer.LexBuilder):
         except ValueError:
             meta = new_metadata(filename, lineno)
             self.errors.append(
-                ParserError(meta, "Attempting to pop absent tag: '{}'".format(tag), None))
+                ParserError(meta, "Attempting to pop absent tag: '{}'".format(tag))
+            )
 
     def pushmeta(self, filename, lineno, key_value):
         """Set a metadata field on the current key-value pairs to be added to transactions.
@@ -307,9 +344,8 @@ class Builder(lexer.LexBuilder):
         except IndexError:
             meta = new_metadata(filename, lineno)
             self.errors.append(
-                ParserError(meta,
-                            "Attempting to pop absent metadata key: '{}'".format(key),
-                            None))
+                ParserError(meta, "Attempting to pop absent metadata key: '{}'".format(key))
+            )
 
     def option(self, filename, lineno, key, value):
         """Process an option directive.
@@ -322,13 +358,11 @@ class Builder(lexer.LexBuilder):
         """
         if key not in self.options:
             meta = new_metadata(filename, lineno)
-            self.errors.append(
-                ParserError(meta, "Invalid option: '{}'".format(key), None))
+            self.errors.append(ParserError(meta, "Invalid option: '{}'".format(key)))
 
         elif key in options.READ_ONLY_OPTIONS:
             meta = new_metadata(filename, lineno)
-            self.errors.append(
-                ParserError(meta, "Option '{}' may not be set".format(key), None))
+            self.errors.append(ParserError(meta, "Option '{}' may not be set".format(key)))
 
         else:
             option_descriptor = options.OPTIONS[key]
@@ -337,8 +371,7 @@ class Builder(lexer.LexBuilder):
             if option_descriptor.deprecated:
                 assert isinstance(option_descriptor.deprecated, str), "Internal error."
                 meta = new_metadata(filename, lineno)
-                self.errors.append(
-                    DeprecatedError(meta, option_descriptor.deprecated, None))
+                self.errors.append(DeprecatedError(meta, option_descriptor.deprecated))
 
             # Rename the option if it has an alias.
             if option_descriptor.alias:
@@ -352,9 +385,8 @@ class Builder(lexer.LexBuilder):
                 except ValueError as exc:
                     meta = new_metadata(filename, lineno)
                     self.errors.append(
-                        ParserError(meta,
-                                    "Error for option '{}': {}".format(key, exc),
-                                    None))
+                        ParserError(meta, "Error for option '{}': {}".format(key, exc))
+                    )
                     return
 
             option = self.options[key]
@@ -366,8 +398,8 @@ class Builder(lexer.LexBuilder):
                 # Set to a dict of values.
                 if not (isinstance(value, tuple) and len(value) == 2):
                     self.errors.append(
-                        ParserError(
-                            meta, "Error for option '{}': {}".format(key, value), None))
+                        ParserError(meta, "Error for option '{}': {}".format(key, value))
+                    )
                     return
                 dict_key, dict_value = value
                 option[dict_key] = dict_value
@@ -375,7 +407,7 @@ class Builder(lexer.LexBuilder):
             elif isinstance(option, bool):
                 # Convert to a boolean.
                 if not isinstance(value, bool):
-                    value = (value.lower() in {'true', 'on'}) or (value == '1')
+                    value = (value.lower() in {"true", "on"}) or (value == "1")
                 self.options[key] = value
 
             else:
@@ -383,7 +415,7 @@ class Builder(lexer.LexBuilder):
                 self.options[key] = value
 
             # Refresh the list of valid account regexps as we go along.
-            if key.startswith('name_'):
+            if key.startswith("name_"):
                 # Update the set of valid account types.
                 self.account_regexp = valid_account_regexp(self.options)
 
@@ -395,7 +427,7 @@ class Builder(lexer.LexBuilder):
           lineno: current line number.
           include_name: A string, the name of the file to include.
         """
-        self.options['include'].append(include_filename)
+        self.options["include"].append(include_filename)
 
     def plugin(self, filename, lineno, plugin_name, plugin_config):
         """Process a plugin directive.
@@ -407,7 +439,7 @@ class Builder(lexer.LexBuilder):
           plugin_config: A string or None, an optional configuration string to
             pass in to the plugin module.
         """
-        self.options['plugin'].append((plugin_name, plugin_config))
+        self.options["plugin"].append((plugin_name, plugin_config))
 
     def amount(self, filename, lineno, number, currency):
         """Process an amount grammar rule.
@@ -462,8 +494,9 @@ class Builder(lexer.LexBuilder):
         """
         if not cost_comp_list:
             return CostSpec(MISSING, None, MISSING, None, None, False)
-        assert isinstance(cost_comp_list, list), (
-            "Internal error in parser: {}".format(cost_comp_list))
+        assert isinstance(cost_comp_list, list), "Internal error in parser: {}".format(
+            cost_comp_list
+        )
 
         compound_cost = None
         date_ = None
@@ -475,37 +508,54 @@ class Builder(lexer.LexBuilder):
                     compound_cost = comp
                 else:
                     self.errors.append(
-                        ParserError(new_metadata(filename, lineno),
-                                    "Duplicate cost: '{}'.".format(comp), None))
+                        ParserError(
+                            new_metadata(filename, lineno),
+                            "Duplicate cost: '{}'.".format(comp),
+                        )
+                    )
 
             elif isinstance(comp, date):
                 if date_ is None:
                     date_ = comp
                 else:
                     self.errors.append(
-                        ParserError(new_metadata(filename, lineno),
-                                    "Duplicate date: '{}'.".format(comp), None))
+                        ParserError(
+                            new_metadata(filename, lineno),
+                            "Duplicate date: '{}'.".format(comp),
+                        )
+                    )
 
             elif comp is MERGE_COST:
                 if merge is None:
                     merge = True
                     self.errors.append(
-                        ParserError(new_metadata(filename, lineno),
-                                    "Cost merging is not supported yet", None))
+                        ParserError(
+                            new_metadata(filename, lineno),
+                            "Cost merging is not supported yet",
+                        )
+                    )
                 else:
                     self.errors.append(
-                        ParserError(new_metadata(filename, lineno),
-                                    "Duplicate merge-cost spec", None))
+                        ParserError(
+                            new_metadata(filename, lineno),
+                            "Duplicate merge-cost spec",
+                        )
+                    )
 
             else:
-                assert isinstance(comp, str), (
-                    "Currency component is not string: '{}'".format(comp))
+                assert isinstance(
+                    comp, str
+                ), "Currency component is not string: '{}'".format(comp)
                 if label is None:
                     label = comp
                 else:
                     self.errors.append(
-                        ParserError(new_metadata(filename, lineno),
-                                    "Duplicate label: '{}'.".format(comp), None))
+                        ParserError(
+                            new_metadata(filename, lineno),
+                            "Duplicate label: '{}'.".format(comp),
+                            None,
+                        )
+                    )
 
         # If there was a cost_comp_list, thus a "{...}" cost basis spec, you must
         # indicate that by creating a CompoundAmount(), always.
@@ -519,9 +569,12 @@ class Builder(lexer.LexBuilder):
                     self.errors.append(
                         ParserError(
                             new_metadata(filename, lineno),
-                            ("Per-unit cost may not be specified using total cost "
-                             "syntax: '{}'; ignoring per-unit cost").format(compound_cost),
-                            None))
+                            (
+                                "Per-unit cost may not be specified using total cost "
+                                "syntax: '{}'; ignoring per-unit cost"
+                            ).format(compound_cost),
+                        )
+                    )
                     # Ignore per-unit number.
                     number_per = ZERO
                 else:
@@ -572,16 +625,16 @@ class Builder(lexer.LexBuilder):
             except KeyError:
                 # If the per-account method is invalid, set it to the global
                 # default method and continue.
-                booking = self.options['booking_method']
+                booking = self.options["booking_method"]
                 error = True
         else:
             booking = None
 
         entry = Open(meta, date, account, currencies, booking)
         if error:
-            self.errors.append(ParserError(meta,
-                                           "Invalid booking method: {}".format(booking_str),
-                                           entry))
+            self.errors.append(
+                ParserError(meta, "Invalid booking method: {}".format(booking_str), entry)
+            )
         return entry
 
     def close(self, filename, lineno, date, account, kvlist):
@@ -717,8 +770,9 @@ class Builder(lexer.LexBuilder):
         tags, links = self._finalize_tags_links(tags_links.tags, tags_links.links)
         return Note(meta, date, account, comment, tags, links)
 
-    def document(self, filename, lineno, date, account, document_filename, tags_links,
-                 kvlist):
+    def document(
+        self, filename, lineno, date, account, document_filename, tags_links, kvlist
+    ):
         """Process a document directive.
 
         Args:
@@ -734,8 +788,9 @@ class Builder(lexer.LexBuilder):
         """
         meta = new_metadata(filename, lineno, kvlist)
         if not path.isabs(document_filename):
-            document_filename = path.abspath(path.join(path.dirname(filename),
-                                                       document_filename))
+            document_filename = path.abspath(
+                path.join(path.dirname(filename), document_filename)
+            )
         tags, links = self._finalize_tags_links(tags_links.tags, tags_links.links)
         return Document(meta, date, account, document_filename, tags, links)
 
@@ -803,11 +858,15 @@ class Builder(lexer.LexBuilder):
         # Prices may not be negative.
         if price and isinstance(price.number, Decimal) and price.number < ZERO:
             self.errors.append(
-                ParserError(meta, (
-                    "Negative prices are not allowed: {} "
-                    "(see http://furius.ca/beancount/doc/bug-negative-prices "
-                    "for workaround)"
-                ).format(price), None))
+                ParserError(
+                    meta,
+                    (
+                        "Negative prices are not allowed: {} "
+                        "(see http://furius.ca/beancount/doc/bug-negative-prices "
+                        "for workaround)"
+                    ).format(price),
+                )
+            )
             # Fix it and continue.
             price = Amount(abs(price.number), price.currency)
 
@@ -818,16 +877,19 @@ class Builder(lexer.LexBuilder):
                 # Note: we could potentially do a better job and attempt to fix
                 # this up after interpolation, but this syntax is pretty rare
                 # anyway.
-                self.errors.append(ParserError(
-                    meta, ("Total price on a posting without units: {}.").format(price),
-                    None))
+                self.errors.append(
+                    ParserError(
+                        meta,
+                        ("Total price on a posting without units: {}.").format(price),
+                    )
+                )
                 price = None
             else:
                 price_number = price.number
                 if price_number is not MISSING:
-                    price_number = (ZERO
-                                    if units.number == ZERO
-                                    else price_number/abs(units.number))
+                    price_number = (
+                        ZERO if units.number == ZERO else price_number / abs(units.number)
+                    )
                     price = Amount(price_number, price.currency)
 
         # Note: Allow zero prices because we need them for round-trips for
@@ -839,15 +901,21 @@ class Builder(lexer.LexBuilder):
 
         # If both cost and price are specified, the currencies must match, or
         # that is an error.
-        if (cost is not None and
-            price is not None and
-            isinstance(cost.currency, str) and
-            isinstance(price.currency, str) and
-            cost.currency != price.currency):
+        if (
+            cost is not None
+            and price is not None
+            and isinstance(cost.currency, str)
+            and isinstance(price.currency, str)
+            and cost.currency != price.currency
+        ):
             self.errors.append(
-                ParserError(meta,
-                            "Cost and price currencies must match: {} != {}".format(
-                                cost.currency, price.currency), None))
+                ParserError(
+                    meta,
+                    "Cost and price currencies must match: {} != {}".format(
+                        cost.currency, price.currency
+                    ),
+                )
+            )
 
         return Posting(account, units, cost, price, chr(flag) if flag else None, meta)
 
@@ -902,9 +970,11 @@ class Builder(lexer.LexBuilder):
             payee, narration = None, ""
         else:
             self.errors.append(
-                ParserError(meta,
-                            "Too many strings on transaction description: {}".format(
-                                txn_strings), None))
+                ParserError(
+                    meta,
+                    "Too many strings on transaction description: {}".format(txn_strings),
+                )
+            )
             return None
         return payee, narration
 
@@ -919,11 +989,14 @@ class Builder(lexer.LexBuilder):
         """
         if self.tags:
             tags.update(self.tags)
-        return (frozenset(tags) if tags else EMPTY_SET,
-                frozenset(links) if links else EMPTY_SET)
+        return (
+            frozenset(tags) if tags else EMPTY_SET,
+            frozenset(links) if links else EMPTY_SET,
+        )
 
-    def transaction(self, filename, lineno, date, flag, txn_strings, tags_links,
-                    posting_or_kv_list):
+    def transaction(
+        self, filename, lineno, date, flag, txn_strings, tags_links, posting_or_kv_list
+    ):
         """Process a transaction directive.
 
         All the postings of the transaction are available at this point, and so the
@@ -960,33 +1033,48 @@ class Builder(lexer.LexBuilder):
                     last_posting = posting_or_kv
                 elif isinstance(posting_or_kv, TagsLinks):
                     if postings:
-                        self.errors.append(ParserError(
-                            meta,
-                            "Tags or links not allowed after first " +
-                            "Posting: {}".format(posting_or_kv), None))
+                        self.errors.append(
+                            ParserError(
+                                meta,
+                                "Tags or links not allowed after first "
+                                + "Posting: {}".format(posting_or_kv),
+                            )
+                        )
                     else:
                         tags.update(posting_or_kv.tags)
                         links.update(posting_or_kv.links)
                 else:
                     if last_posting is None:
-                        value = explicit_meta.setdefault(posting_or_kv.key,
-                                                         posting_or_kv.value)
+                        value = explicit_meta.setdefault(
+                            posting_or_kv.key, posting_or_kv.value
+                        )
                         if value is not posting_or_kv.value:
-                            self.errors.append(ParserError(
-                                meta, "Duplicate metadata field on entry: {}".format(
-                                    posting_or_kv), None))
+                            self.errors.append(
+                                ParserError(
+                                    meta,
+                                    "Duplicate metadata field on entry: {}".format(
+                                        posting_or_kv
+                                    ),
+                                )
+                            )
                     else:
                         if last_posting.meta is None:
                             last_posting = last_posting._replace(meta={})
                             postings.pop(-1)
                             postings.append(last_posting)
 
-                        value = last_posting.meta.setdefault(posting_or_kv.key,
-                                                             posting_or_kv.value)
+                        value = last_posting.meta.setdefault(
+                            posting_or_kv.key, posting_or_kv.value
+                        )
                         if value is not posting_or_kv.value:
-                            self.errors.append(ParserError(
-                                meta, "Duplicate posting metadata field: {}".format(
-                                    posting_or_kv), None))
+                            self.errors.append(
+                                ParserError(
+                                    meta,
+                                    "Duplicate posting metadata field: {}".format(
+                                        posting_or_kv
+                                    ),
+                                )
+                            )
 
         # Freeze the tags & links or set to default empty values.
         tags, links = self._finalize_tags_links(tags, links)
@@ -1023,5 +1111,4 @@ class Builder(lexer.LexBuilder):
             postings = []
 
         # Create the transaction.
-        return Transaction(meta, date, chr(flag),
-                           payee, narration, tags, links, postings)
+        return Transaction(meta, date, chr(flag), payee, narration, tags, links, postings)
