@@ -1,6 +1,8 @@
-"""Conversion from internal data structures to text.
-"""
-__copyright__ = "Copyright (C) 2014-2018  Martin Blais"
+"""Conversion from internal data structures to text."""
+
+from __future__ import annotations
+
+__copyright__ = "Copyright (C) 2014-2025  Martin Blais"
 __license__ = "GNU GPLv2"
 
 import codecs
@@ -13,14 +15,14 @@ import textwrap
 from decimal import Decimal
 from typing import Optional
 
-from beancount.core import position
-from beancount.core import convert
-from beancount.core import inventory
-from beancount.core import amount
 from beancount.core import account
+from beancount.core import amount
+from beancount.core import convert
 from beancount.core import data
-from beancount.core import interpolate
 from beancount.core import display_context
+from beancount.core import interpolate
+from beancount.core import inventory
+from beancount.core import position
 from beancount.utils import misc_utils
 
 
@@ -55,7 +57,7 @@ def align_position_strings(strings):
     max_unknown = 0
 
     string_items = []
-    search = re.compile('[A-Z]').search
+    search = re.compile("[A-Z]").search
     for string in strings:
         match = search(string)
         if match:
@@ -76,10 +78,9 @@ def align_position_strings(strings):
     fmt_unknown = "{{:<{0}}}".format(max_total).format
 
     # Align the strings and return them.
-    # pylint: disable=format-string-without-interpolation
+
     aligned_strings = []
     for index, string in string_items:
-        # pylint: disable=format-string-without-interpolation
         if index is not None:
             string = fmt(string[:index], string[index:])
         else:
@@ -102,16 +103,21 @@ class EntryPrinter:
         string for printout.
     """
 
-    # pylint: disable=invalid-name
-
-    def __init__(self, dcontext=None, render_weight=False, min_width_account=None,
-                 prefix=None, stringify_invalid_types=False, write_source=False):
+    def __init__(
+        self,
+        dcontext=None,
+        render_weight=False,
+        min_width_account=None,
+        prefix=None,
+        stringify_invalid_types=False,
+        write_source=False,
+    ):
         self.dcontext = dcontext or display_context.DEFAULT_DISPLAY_CONTEXT
         self.dformat = self.dcontext.build(precision=display_context.Precision.MOST_COMMON)
         self.dformat_max = self.dcontext.build(precision=display_context.Precision.MAXIMUM)
         self.render_weight = render_weight
         self.min_width_account = min_width_account
-        self.prefix = prefix or '  '
+        self.prefix = prefix or "  "
         self.stringify_invalid_types = stringify_invalid_types
         self.write_source = write_source
 
@@ -132,7 +138,7 @@ class EntryPrinter:
         method(obj, oss)
         return oss.getvalue()
 
-    META_IGNORE = {'filename', 'lineno'}
+    META_IGNORE = {"filename", "lineno"}
 
     def write_metadata(self, meta, oss, prefix=None):
         """Write metadata to the file object, excluding filename and line number.
@@ -149,18 +155,18 @@ class EntryPrinter:
         # Note: meta.items() is assumed stable from 3.7 onwards; we're not sorting
         # on purpose in order to keep the original insertion order in print.
         for key, value in meta.items():
-            if key not in self.META_IGNORE and not key.startswith('__'):
+            if key not in self.META_IGNORE and not key.startswith("__"):
                 value_str = None
                 if isinstance(value, str):
                     value_str = '"{}"'.format(misc_utils.escape_string(value))
                 elif isinstance(value, (Decimal, datetime.date, amount.Amount, enum.Enum)):
                     value_str = str(value)
                 elif isinstance(value, bool):
-                    value_str = 'TRUE' if value else 'FALSE'
+                    value_str = "TRUE" if value else "FALSE"
                 elif isinstance(value, (dict, inventory.Inventory)):
-                    pass # Ignore dicts, don't print them out.
+                    pass  # Ignore dicts, don't print them out.
                 elif value is None:
-                    value_str = ''  # Render null metadata as empty, on purpose.
+                    value_str = ""  # Render null metadata as empty, on purpose.
                 else:
                     if self.stringify_invalid_types:
                         # This is only intended to be used during development,
@@ -189,7 +195,7 @@ class EntryPrinter:
         if prefix is None:
             prefix = self.prefix
 
-        oss.write('{}; source: {}\n'.format(prefix, render_source(meta)))
+        oss.write("{}; source: {}\n".format(prefix, render_source(meta)))
 
     def Transaction(self, entry, oss):
         # Compute the string for the payee and narration line.
@@ -204,48 +210,57 @@ class EntryPrinter:
 
         if entry.tags:
             for tag in sorted(entry.tags):
-                strings.append('#{}'.format(tag))
+                strings.append("#{}".format(tag))
         if entry.links:
             for link in sorted(entry.links):
-                strings.append('^{}'.format(link))
+                strings.append("^{}".format(link))
 
-        oss.write('{e.date} {flag} {}\n'.format(' '.join(strings), e=entry,
-                                                flag=render_flag(entry.flag)))
+        oss.write(
+            "{e.date} {flag} {}\n".format(
+                " ".join(strings), e=entry, flag=render_flag(entry.flag)
+            )
+        )
         self.write_metadata(entry.meta, oss)
 
-        rows = [self.render_posting_strings(posting)
-                for posting in entry.postings]
+        rows = [self.render_posting_strings(posting) for posting in entry.postings]
         strs_account = [row[0] for row in rows]
-        width_account = (max(len(flag_account) for flag_account in strs_account)
-                         if strs_account
-                         else 1)
+        width_account = (
+            max(len(flag_account) for flag_account in strs_account) if strs_account else 1
+        )
         strs_position, width_position = align_position_strings(row[1] for row in rows)
         strs_weight, width_weight = align_position_strings(row[2] for row in rows)
 
         if self.min_width_account and self.min_width_account > width_account:
             width_account = self.min_width_account
 
-        non_trivial_balance = (any(map(interpolate.has_nontrivial_balance, entry.postings))
-                               if self.render_weight and width_weight > 0
-                               else False)
+        non_trivial_balance = (
+            any(map(interpolate.has_nontrivial_balance, entry.postings))
+            if self.render_weight and width_weight > 0
+            else False
+        )
         if non_trivial_balance:
-            for posting, account, position, weight in zip(entry.postings,
-                                                          strs_account,
-                                                          strs_position,
-                                                          strs_weight):
-                oss.write(f"{self.prefix}{account:{width_account}}  "
-                          f"{position:{width_position}}  "
-                          f"; {weight:{max(1, width_weight)}}".rstrip() + '\n')
+            for posting, account, position, weight in zip(
+                entry.postings, strs_account, strs_position, strs_weight
+            ):
+                oss.write(
+                    f"{self.prefix}{account:{width_account}}  "
+                    f"{position:{width_position}}  "
+                    f"; {weight:{max(1, width_weight)}}".rstrip()
+                    + "\n"
+                )
                 if posting.meta:
-                    self.write_metadata(posting.meta, oss, '    ')
+                    self.write_metadata(posting.meta, oss, "    ")
         else:
-            for posting, account, position in zip(entry.postings,
-                                                  strs_account,
-                                                  strs_position):
-                oss.write(f"{self.prefix}{account:{width_account}}  "
-                          f"{position:{max(1, width_position)}}".rstrip() + '\n')
+            for posting, account, position in zip(
+                entry.postings, strs_account, strs_position
+            ):
+                oss.write(
+                    f"{self.prefix}{account:{width_account}}  "
+                    f"{position:{max(1, width_position)}}".rstrip()
+                    + "\n"
+                )
                 if posting.meta:
-                    self.write_metadata(posting.meta, oss, '    ')
+                    self.write_metadata(posting.meta, oss, "    ")
 
     def render_posting_strings(self, posting):
         """This renders the three components of a posting: the account and its optional
@@ -261,23 +276,25 @@ class EntryPrinter:
             weight_str: A string, the rendered weight of the posting.
         """
         # Render a string of the flag and the account.
-        flag = '{} '.format(render_flag(posting.flag)) if posting.flag else ''
+        flag = "{} ".format(render_flag(posting.flag)) if posting.flag else ""
         flag_account = flag + posting.account
 
         # Render a string with the amount and cost and optional price, if
         # present. Also render a string with the weight.
-        weight_str = ''
+        weight_str = ""
         if isinstance(posting.units, amount.Amount):
             position_str = position.to_string(posting, self.dformat)
             # Note: we render weights at maximum precision, for debugging.
-            if posting.cost is None or (isinstance(posting.cost, position.Cost) and
-                                        isinstance(posting.cost.number, Decimal)):
+            if posting.cost is None or (
+                isinstance(posting.cost, position.Cost)
+                and isinstance(posting.cost.number, Decimal)
+            ):
                 weight_str = str(convert.get_weight(posting))
         else:
-            position_str = ''
+            position_str = ""
 
         if posting.price is not None:
-            position_str += ' @ {}'.format(posting.price.to_string(self.dformat_max))
+            position_str += " @ {}".format(posting.price.to_string(self.dformat_max))
 
         return flag_account, position_str, weight_str
 
@@ -286,80 +303,95 @@ class EntryPrinter:
         # method rendering a transaction attempts to align the posting strings
         # together.
         flag_account, position_str, weight_str = self.render_posting_strings(posting)
-        oss.write('{}{:64} {} ; {}\n'.format(self.prefix,
-                                             flag_account,
-                                             position_str,
-                                             weight_str).rstrip())
+        oss.write(
+            "{}{:64} {} ; {}\n".format(
+                self.prefix, flag_account, position_str, weight_str
+            ).rstrip()
+        )
         if posting.meta:
-            self.write_metadata(posting.meta, oss, '    ')
+            self.write_metadata(posting.meta, oss, "    ")
 
     def Balance(self, entry, oss):
-        comment = '   ; Diff: {}'.format(entry.diff_amount) if entry.diff_amount else ''
-        number_str = (self.dformat.format(entry.amount.number, entry.amount.currency)
-                      if isinstance(entry.amount.number, Decimal)
-                      else str(self.number))
+        comment = "   ; Diff: {}".format(entry.diff_amount) if entry.diff_amount else ""
+        number_str = (
+            self.dformat.format(entry.amount.number, entry.amount.currency)
+            if isinstance(entry.amount.number, Decimal)
+            else str(self.number)
+        )
 
         # Render optional tolerance.
-        tolerance = ''
+        tolerance = ""
         if entry.tolerance:
             tolerance_fmt = self.dformat.format(entry.tolerance, entry.amount.currency)
-            tolerance = '~ {tolerance} '.format(tolerance=tolerance_fmt)
+            tolerance = "~ {tolerance} ".format(tolerance=tolerance_fmt)
 
-        oss.write(('{e.date} balance {e.account:47} {amount} {tolerance}{currency}'
-                   '{comment}\n').format(e=entry,
-                                         amount=number_str,
-                                         tolerance=tolerance,
-                                         currency=entry.amount.currency,
-                                         comment=comment))
+        oss.write(
+            (
+                "{e.date} balance {e.account:47} {amount} {tolerance}{currency}"
+                "{comment}\n"
+            ).format(
+                e=entry,
+                amount=number_str,
+                tolerance=tolerance,
+                currency=entry.amount.currency,
+                comment=comment,
+            )
+        )
         self.write_metadata(entry.meta, oss)
 
     def Note(self, entry, oss):
         oss.write('{e.date} note {e.account} "{e.comment}"'.format(e=entry))
         if entry.tags or entry.links:
-            oss.write(' ')
+            oss.write(" ")
             for tag in sorted(entry.tags):
-                oss.write('#{}'.format(tag))
+                oss.write("#{}".format(tag))
             for link in sorted(entry.links):
-                oss.write('^{}'.format(link))
-        oss.write('\n')
+                oss.write("^{}".format(link))
+        oss.write("\n")
         self.write_metadata(entry.meta, oss)
 
     def Document(self, entry, oss):
         oss.write('{e.date} document {e.account} "{e.filename}"'.format(e=entry))
         if entry.tags or entry.links:
-            oss.write(' ')
+            oss.write(" ")
             for tag in sorted(entry.tags):
-                oss.write('#{}'.format(tag))
+                oss.write("#{}".format(tag))
             for link in sorted(entry.links):
-                oss.write('^{}'.format(link))
-        oss.write('\n')
+                oss.write("^{}".format(link))
+        oss.write("\n")
         self.write_metadata(entry.meta, oss)
 
     def Pad(self, entry, oss):
-        oss.write('{e.date} pad {e.account} {e.source_account}\n'.format(e=entry))
+        oss.write("{e.date} pad {e.account} {e.source_account}\n".format(e=entry))
         self.write_metadata(entry.meta, oss)
 
     def Open(self, entry, oss):
-        oss.write('{e.date} open {e.account:47} {currencies} {booking}'.format(
-            e=entry,
-            currencies=','.join(entry.currencies or []),
-            booking=('"{}"'.format(entry.booking.name)
-                     if entry.booking is not None
-                     else '')).rstrip())
-        oss.write('\n')
+        oss.write(
+            "{e.date} open {e.account:47} {currencies} {booking}".format(
+                e=entry,
+                currencies=",".join(entry.currencies or []),
+                booking=(
+                    '"{}"'.format(entry.booking.name) if entry.booking is not None else ""
+                ),
+            ).rstrip()
+        )
+        oss.write("\n")
         self.write_metadata(entry.meta, oss)
 
     def Close(self, entry, oss):
-        oss.write('{e.date} close {e.account}\n'.format(e=entry))
+        oss.write("{e.date} close {e.account}\n".format(e=entry))
         self.write_metadata(entry.meta, oss)
 
     def Commodity(self, entry, oss):
-        oss.write('{e.date} commodity {e.currency}\n'.format(e=entry))
+        oss.write("{e.date} commodity {e.currency}\n".format(e=entry))
         self.write_metadata(entry.meta, oss)
 
     def Price(self, entry, oss):
-        oss.write('{e.date} price {e.currency:<22} {amount:>22}\n'.format(
-            e=entry, amount=entry.amount.to_string(self.dformat_max)))
+        oss.write(
+            "{e.date} price {e.currency:<22} {amount:>22}\n".format(
+                e=entry, amount=entry.amount.to_string(self.dformat_max)
+            )
+        )
         self.write_metadata(entry.meta, oss)
 
     def Event(self, entry, oss):
@@ -374,7 +406,7 @@ class EntryPrinter:
         custom_values = []
         for value, dtype in entry.values:
             if dtype is account.TYPE:
-                value = '{}'.format(value)
+                value = "{}".format(value)
             elif isinstance(value, str):
                 value = '"{}"'.format(value)
             elif isinstance(value, Decimal):
@@ -382,24 +414,30 @@ class EntryPrinter:
             elif isinstance(value, datetime.date):
                 value = value.isoformat()
             elif isinstance(value, bool):
-                value = 'TRUE' if value else 'FALSE'
+                value = "TRUE" if value else "FALSE"
             elif isinstance(value, amount.Amount):
                 value = value.to_string()
             custom_values.append(value)
-        oss.write('{e.date} custom "{e.type}" {}\n'.format(" ".join(custom_values),
-                                                           e=entry))
+        oss.write(
+            '{e.date} custom "{e.type}" {}\n'.format(" ".join(custom_values), e=entry)
+        )
         self.write_metadata(entry.meta, oss)
 
 
 def render_flag(inflag: Optional[str]) -> str:
     """Render a flag, which can be None, a symbol of a character to a string."""
     if not inflag:
-        return ''
+        return ""
     return inflag
 
 
-def format_entry(entry, dcontext=None, render_weights=False, prefix=None,
-                 write_source=False):
+def format_entry(
+    entry: data.Directive,
+    dcontext: display_context.DisplayContext | None = None,
+    render_weights: bool = False,
+    prefix: str | None = None,
+    write_source: bool = False,
+) -> str:
     """Format an entry into a string in the same input syntax the parser accepts.
 
     Args:
@@ -414,12 +452,12 @@ def format_entry(entry, dcontext=None, render_weights=False, prefix=None,
     Returns:
       A string, the formatted entry.
     """
-    return EntryPrinter(dcontext, render_weights, prefix=prefix,
-                        write_source=write_source)(entry)
+    return EntryPrinter(dcontext, render_weights, prefix=prefix, write_source=write_source)(
+        entry
+    )
 
 
-def print_entry(entry, dcontext=None, render_weights=False, file=None,
-                write_source=False):
+def print_entry(entry, dcontext=None, render_weights=False, file=None, write_source=False):
     """A convenience function that prints a single entry to a file.
 
     Args:
@@ -434,19 +472,21 @@ def print_entry(entry, dcontext=None, render_weights=False, file=None,
     """
     # TODO(blais): DO remove this now, it's a huge annoyance not to be able to
     # print in-between other statements.
-    output = file or (codecs.getwriter("utf-8")(sys.stdout.buffer)
-                      if hasattr(sys.stdout, 'buffer') else
-                      sys.stdout)
-    output.write(format_entry(entry, dcontext, render_weights,
-                              write_source=write_source))
-    output.write('\n')
+    output = file or (
+        codecs.getwriter("utf-8")(sys.stdout.buffer)
+        if hasattr(sys.stdout, "buffer")
+        else sys.stdout
+    )
+    output.write(format_entry(entry, dcontext, render_weights, write_source=write_source))
+    output.write("\n")
 
 
 # TODO(blais): Change this to a function which accepts the same optional
 # arguments as the printer object. Isolate the spacer/segmentation algorithm to
 # its own function.
-def print_entries(entries, dcontext=None, render_weights=False, file=None, prefix=None,
-                  write_source=False):
+def print_entries(
+    entries, dcontext=None, render_weights=False, file=None, prefix=None, write_source=False
+):
     """A convenience function that prints a list of entries to a file.
 
     Args:
@@ -461,9 +501,11 @@ def print_entries(entries, dcontext=None, render_weights=False, file=None, prefi
         especially in a multi-file setup
     """
     assert isinstance(entries, list), "Entries is not a list: {}".format(entries)
-    output = file or (codecs.getwriter("utf-8")(sys.stdout.buffer)
-                      if hasattr(sys.stdout, 'buffer') else
-                      sys.stdout)
+    output = file or (
+        codecs.getwriter("utf-8")(sys.stdout.buffer)
+        if hasattr(sys.stdout, "buffer")
+        else sys.stdout
+    )
 
     if prefix:
         output.write(prefix)
@@ -473,9 +515,12 @@ def print_entries(entries, dcontext=None, render_weights=False, file=None, prefi
         # Insert a newline between transactions and between blocks of directives
         # of the same type.
         entry_type = type(entry)
-        if (entry_type in (data.Transaction, data.Commodity) or
-            entry_type is not previous_type or write_source):
-            output.write('\n')
+        if (
+            entry_type in (data.Transaction, data.Commodity)
+            or entry_type is not previous_type
+            or write_source
+        ):
+            output.write("\n")
             previous_type = entry_type
 
         string = eprinter(entry)
@@ -494,7 +539,7 @@ def render_source(meta):
       A string, rendered to be interpretable as a message location for Emacs or
       other editors.
     """
-    return '{}:{:8}'.format(meta['filename'], '{}:'.format(meta['lineno']))
+    return "{}:{}".format(meta["filename"], "{}:".format(meta["lineno"]))
 
 
 def format_error(error):
@@ -508,13 +553,13 @@ def format_error(error):
       A string, the errors rendered.
     """
     oss = io.StringIO()
-    oss.write('{} {}\n'.format(render_source(error.source), error.message))
+    oss.write("{} {}\n".format(render_source(error.source), error.message))
     if error.entry is not None:
         entries = error.entry if isinstance(error.entry, list) else [error.entry]
-        error_string = '\n'.join(format_entry(entry) for entry in entries)
-        oss.write('\n')
-        oss.write(textwrap.indent(error_string, '   '))
-        oss.write('\n')
+        error_string = "\n".join(format_entry(entry) for entry in entries)
+        oss.write("\n")
+        oss.write(textwrap.indent(error_string, "   "))
+        oss.write("\n")
     return oss.getvalue()
 
 
@@ -527,7 +572,7 @@ def print_error(error, file=None):
     """
     output = file or sys.stdout
     output.write(format_error(error))
-    output.write('\n')
+    output.write("\n")
 
 
 def print_errors(errors, file=None, prefix=None):
@@ -542,4 +587,4 @@ def print_errors(errors, file=None, prefix=None):
         output.write(prefix)
     for error in errors:
         output.write(format_error(error))
-        output.write('\n')
+        output.write("\n")
